<?php
/**
 * @file
 * Module admin page callbacks.
 */

/**
 * Form builder function.
 */
function ldapcontrol_admin_settings() {
  drupal_add_js(drupal_get_path('module', 'ldapcontrol') . '/ldapcontrol.admin.js');

  // General settings.
  $form['general'] = array(
    '#type' => 'fieldset',
    '#title' => t('General settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['general']['ldapcontrol_user'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable LDAP user data handling.'),
    '#default_value' => LDAPCONTROL_USER,
  );
  $form['general']['ldapcontrol_debug'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable debugging'),
    '#description' => t('Show LDAP data in LDIF style for configuration debugging. Access right: "administer ldap modules" is needed.'),
    '#default_value' => LDAPCONTROL_DEBUG,
  );

  // LDAP authentication.
  $form['ldap'] = array(
    '#type' => 'fieldset',
    '#title' => ('LDAP authentication'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#description' => t('Choose LDAP server new accounts will be created in.'),
  );
  $ldap_servers = array();
  $result = db_query("SELECT sid, name, status
    FROM {ldapauth} ORDER BY weight");
  while ($row = db_fetch_object($result)) {
    $ldap_servers[$row->sid] = $row->name;
  }
  $form['ldap']['ldapcontrol_server'] = array(
    '#type' => 'select',
    '#title' => t('LDAP server'),
    '#options' => $ldap_servers,
    '#default_value' => LDAPCONTROL_SERVER,
    '#description' => t('Select LDAP server to create LDAP accounts in.'),
  );
  if (LDAPCONTROL_PASS_CLEAR || !LDAPCONTROL_PASS) {
    $form['ldap']['ldapcontrol_dn'] = array(
      '#type' => 'textfield',
      '#title' => t('Bind DN'),
      '#size' => 50,
      '#maxlength' => 255,
      '#default_value' => LDAPCONTROL_DN,
      '#description' => t('Bind DN should have the rights to create new LDAP entries.'),
    );
    variable_del('ldapcontrol_pass');
    $form['ldap']['ldapcontrol_pass'] = array(
      '#type' => 'password',
      '#title' => t('Password'),
      '#size' => 50,
      '#maxlength' => 255,
      '#default_value' => '',
    );
    variable_set('ldapcontrol_pass_clear', FALSE);
  }
  else {
    $form['ldap']['ldapcontrol_dn'] = array(
      '#type' => 'item',
      '#title' => t('Bind DN'),
      '#value' => LDAPCONTROL_DN,
    );
    $form['ldap']['ldapcontrol_pass_clear'] = array(
      '#type' => 'checkbox',
      '#title' => t('Clear current password and change DN'),
      '#default_value' => FALSE,
    );
  }
  $form['ldap']['test'] = array(
    '#type'  => 'submit',
    '#value' => t('Test'),
    '#suffix' => '<div id="test-spinner" style="display: none;">'
    . theme_image(drupal_get_path('module', 'ldapauth') . '/images/spinner.gif')
    . '</div><div id="test-message" class="messages" style="display: none;"></div>',
  );

  $form['submit'] = array(
    '#type'  => 'submit',
    '#value' => t('Save configuration'),
  );
  $form['reset'] = array(
    '#type'  => 'submit',
    '#value' => t('Reset to defaults'),
  );
  return $form;
}


/**
 * Submit hook for the settings form.
 */
function ldapcontrol_admin_settings_submit($form, &$form_state) {
  $op = $form_state['clicked_button']['#value'];
  switch ($op) {
    case t('Save configuration'):
      $values = $form_state['values'];

      // General settings.
      variable_set('ldapcontrol_user', $values['ldapcontrol_user']);
      variable_set('ldapcontrol_username', $values['ldapcontrol_username']);
      variable_set('ldapcontrol_debug', $values['ldapcontrol_debug']);

      // LDAP authentication.
      variable_set('ldapcontrol_server', $values['ldapcontrol_server']);
      if (!empty($values['ldapcontrol_pass'])) {
        variable_set('ldapcontrol_dn', $values['ldapcontrol_dn']);
        variable_set('ldapcontrol_pass', $values['ldapcontrol_pass']);
      }
      if (isset($values['ldapcontrol_pass_clear']) && $values['ldapcontrol_pass_clear'] == 1) {
        variable_set('ldapcontrol_pass', '');
      }

      drupal_set_message(t('The configuration options have been saved.'));
      break;

    case t('Reset to defaults'):
      $values = $form_state['values'];

      variable_del('ldapcontrol_user');
      variable_del('ldapcontrol_debug');
      variable_del('ldapcontrol_server');
      variable_del('ldapcontrol_dn');
      variable_del('ldapcontrol_pass');
      variable_del('ldapcontrol_pass_clear');

      drupal_set_message(t('The configuration options have been reset to their default values.'));
      break;

    case t('Test'):
      global $_ldapcontrol_ldap;

      // Try to authenticate.
      if (!$_ldapcontrol_ldap->connect(LDAPCONTROL_DN, LDAPCONTROL_PASS)) {
        drupal_set_message(t('Authentication with the LDAP server for the dn %dn and saved password failed.', array('%dn' => LDAPCONTROL_DN)), 'error');
      }
      else {
        drupal_set_message(t('Authentication with the LDAP server for the dn %dn and saved password succeeded.', array('%dn' => LDAPCONTROL_DN)));
      }
      return;
  }
}

/**
 * Implements the AJAX server test.
 */
function _ldapcontrol_ajax_test() {
  global $_ldapcontrol_ldap;

  $sid = check_plain($_POST['sid']);
  if (!is_numeric($sid)) {
    return;
  }

  _ldapcontrol_ldapobject($sid);

  if (check_plain($_POST['bindpw_clear']) == 'undefined' && check_plain($_POST['binddn']) && check_plain($_POST['bindpw'])) {
    $binddn = check_plain($_POST['binddn']);
    $bindpw = check_plain($_POST['bindpw']);
  }
  else {
    $binddn = LDAPCONTROL_DN;
    $bindpw = LDAPCONTROL_PASS;
  }

  drupal_json($_ldapcontrol_ldap->connect($binddn, $bindpw) ? array('status' => 1, 'message' => t('Authentication with the LDAP server succeeded.')) : array('status' => 0, 'message' => t('Authentication with the LDAP server failed.')));
  exit;
}
